/*-
 * ============LICENSE_START=======================================================
 *  Copyright (C) 2020 Nordix Foundation.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * ============LICENSE_END=========================================================
 */
package org.onap.so.etsi.nfvo.ns.lcm.database.service;

import java.util.List;
import java.util.Optional;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.NfvoJob;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.NfvoNfInst;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.NfvoNsInst;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.NsLcmOpOcc;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.OperationStateEnum;
import org.onap.so.etsi.nfvo.ns.lcm.database.beans.State;
import org.onap.so.etsi.nfvo.ns.lcm.database.repository.NSLcmOpOccRepository;
import org.onap.so.etsi.nfvo.ns.lcm.database.repository.NfvoJobRepository;
import org.onap.so.etsi.nfvo.ns.lcm.database.repository.NfvoNfInstRepository;
import org.onap.so.etsi.nfvo.ns.lcm.database.repository.NfvoNsInstRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Waqas Ikram (waqas.ikram@est.tech)
 *
 */

@Service
public class DatabaseServiceProvider {
    private static final Logger logger = LoggerFactory.getLogger(DatabaseServiceProvider.class);

    private final NfvoJobRepository nfvoJobRepository;

    private final NfvoNsInstRepository nfvoNsInstRepository;

    private final NfvoNfInstRepository nfvoNfInstRepository;

    private final NSLcmOpOccRepository nsLcmOpOccRepository;

    @Autowired
    public DatabaseServiceProvider(final NfvoJobRepository nfvoJobRepository,
            final NfvoNsInstRepository nfvoNsInstRepository, final NfvoNfInstRepository nfvoNfInstRepository,
            final NSLcmOpOccRepository nsLcmOpOccRepository) {
        this.nfvoJobRepository = nfvoJobRepository;
        this.nfvoNsInstRepository = nfvoNsInstRepository;
        this.nfvoNfInstRepository = nfvoNfInstRepository;
        this.nsLcmOpOccRepository = nsLcmOpOccRepository;
    }

    public boolean addJob(final NfvoJob job) {
        logger.info("Adding NfvoJob: {} to database", job);
        return nfvoJobRepository.saveAndFlush(job) != null;
    }

    public Optional<NfvoJob> getJob(final String jobId) {
        logger.info("Querying database for NfvoJob using jobId: {}", jobId);
        return nfvoJobRepository.findById(jobId);
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW, isolation = Isolation.REPEATABLE_READ, readOnly = true)
    public Optional<NfvoJob> getRefreshedJob(final String jobId) {
        logger.info("Querying database for NfvoJob using jobId: {}", jobId);
        final Optional<NfvoJob> optional = getJob(jobId);
        if (optional.isPresent()) {
            nfvoJobRepository.refreshEntity(optional.get());
        }
        return optional;
    }

    public Optional<NfvoJob> getJobByResourceId(final String resourceId) {
        logger.info("Querying database for NfvoJob using resourceId: {}", resourceId);
        return nfvoJobRepository.findByResourceId(resourceId);
    }

    public boolean isNsInstExists(final String name) {
        logger.info("Checking if NfvoNsInst entry exists in database using name: {}", name);
        return nfvoNsInstRepository.existsNfvoNsInstByName(name);
    }

    public boolean isNsInstExists(final String nsInstId, final String nfName) {
        logger.info("Checking if NfvoNfInst entry exists in database using nsInstId: {} and nfName: {}", nsInstId,
                nfName);
        return nfvoNfInstRepository.findByNsInstNsInstIdAndName(nsInstId, nfName).isEmpty();
    }

    public boolean saveNfvoNsInst(final NfvoNsInst nfvoNsInst) {
        logger.info("Saving NfvoNsInst: {} to database", nfvoNsInst);
        return nfvoNsInstRepository.saveAndFlush(nfvoNsInst) != null;
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public boolean updateNsInstState(final String nsInstId, final State state) {
        logger.info("Updating NfvoNsInst: {} State to {}", nsInstId, state);
        return nfvoNsInstRepository.updateNsInstState(nsInstId, state) > 0;
    }

    public Optional<NfvoNsInst> getNfvoNsInst(final String nsInstId) {
        logger.info("Querying database for NfvoNsInst using nsInstId: {}", nsInstId);
        return nfvoNsInstRepository.findById(nsInstId);
    }

    public Optional<NfvoNsInst> getNfvoNsInstByName(final String name) {
        logger.info("Querying database for NfvoNsInst using name: {}", name);
        return nfvoNsInstRepository.findByName(name);
    }

    public boolean saveNfvoNfInst(final NfvoNfInst nfvoNfInst) {
        logger.info("Saving NfvoNfInst: {} to database", nfvoNfInst);
        return nfvoNfInstRepository.saveAndFlush(nfvoNfInst) != null;
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public boolean updateNfInstState(final String nfInstId, final State state) {
        logger.info("Updating NfvoNfInst: {} State to {}", nfInstId, state);
        return nfvoNfInstRepository.updateNfInstState(nfInstId, state) > 0;
    }

    public List<NfvoNfInst> getNfvoNfInstByNsInstId(final String nsInstId) {
        logger.info("Querying database for NfvoNfInst using nsInstId: {}", nsInstId);
        return nfvoNfInstRepository.findByNsInstNsInstId(nsInstId);
    }

    public List<NfvoNfInst> getNfvoNfInstByNsInstIdAndNfName(final String nsInstId, final String name) {
        logger.info("Querying database for NfvoNfInst using nsInstId: {} and name : {} ", nsInstId, name);
        return nfvoNfInstRepository.findByNsInstNsInstIdAndName(nsInstId, name);
    }

    public Optional<NfvoNfInst> getNfvoNfInst(final String nfInstId) {
        logger.info("Querying database for NfvoNfInst using nfInstId: {}", nfInstId);
        return nfvoNfInstRepository.findByNfInstId(nfInstId);
    }

    public boolean isNfInstExists(final String nfInstId) {
        logger.info("Checking if NfvoNfInst entry exists in database using nfInstId: {}", nfInstId);
        return nfvoNfInstRepository.findByNfInstId(nfInstId).isPresent();
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void deleteNfvoNfInst(final String nfInstId) {
        logger.info("Deleting NfvoNfInst with nfInstId: {} from database", nfInstId);
        nfvoNfInstRepository.deleteNfvoNfInstUsingNfInstId(nfInstId);
    }

    public boolean addNSLcmOpOcc(final NsLcmOpOcc nsLcmOpOcc) {
        logger.info("Adding NSLcmOpOcc: {} to database", nsLcmOpOcc);
        return nsLcmOpOccRepository.saveAndFlush(nsLcmOpOcc) != null;
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public boolean updateNsLcmOpOccOperationState(final String id, final OperationStateEnum operationState) {
        logger.info("Updating NsLcmOpOcc: {} operationState to {}", id, operationState);
        return nsLcmOpOccRepository.updateNsLcmOpOccOperationState(id, operationState) > 0;
    }

    public Optional<NsLcmOpOcc> getNsLcmOpOcc(final String id) {
        logger.info("Querying database for NsLcmOpOcc using id: {}", id);
        return nsLcmOpOccRepository.findById(id);
    }

    public void deleteNfvoNsInst(final String nsInstId) {
        logger.info("Deleting NfvoNsInst with nsInstId: {}", nsInstId);
        nfvoNsInstRepository.deleteById(nsInstId);
    }
}
